@@ -484,24 +484,14 @@ impl OperatorValidator {
                 self.check_reference_types_enabled()
             }
             TypeOrFuncType::Type(Type::V128) => self.check_simd_enabled(),
-            TypeOrFuncType::FuncType(idx) => {
-                let ty = func_type_at(&resources, idx)?;
-                if !self.features.multi_value {
-                    if ty.len_outputs() > 1 {
-                        return Err(OperatorValidatorError::new(
-                            "blocks, loops, and ifs may only return at most one \
-                             value when multi-value is not enabled",
-                        ));
-                    }
-                    if ty.len_inputs() > 0 {
-                        return Err(OperatorValidatorError::new(
-                            "blocks, loops, and ifs accept no parameters \
-                             when multi-value is not enabled",
-                        ));
-                    }
-                }
-                Ok(())
+            TypeOrFuncType::FuncType(_) if !self.features.multi_value => {
+                return Err(OperatorValidatorError::new(
+                    "blocks, loops, and ifs may only produce a resulttype when multi-value is not \
+                    enabled",
+                ));
             }
+
+            TypeOrFuncType::FuncType(idx) => func_type_at(&resources, idx).map(|_| ()),
             _ => Err(OperatorValidatorError::new("invalid block return type")),
         }
     }
@@ -581,7 +571,7 @@ impl OperatorValidator {
         operator: &Operator,
         resources: &impl WasmModuleResources,
     ) -> OperatorValidatorResult<()> {
-        if self.control.len() == 0 {
+        if self.control.is_empty() {
             bail_op_err!("operators remaining after end of function");
         }
         match *operator {
@@ -704,14 +694,10 @@ impl OperatorValidator {
                 // now, but it's used to allow for `if` statements that are
                 // missing an `else` block which have the same parameter/return
                 // types on the block (since that's valid).
-                match frame.kind {
-                    FrameKind::If => {
-                        self.push_ctrl(FrameKind::Else, frame.block_type, resources)?;
-                        frame = self.pop_ctrl(resources)?;
-                    }
-                    _ => (),
+                if frame.kind == FrameKind::If {
+                    self.push_ctrl(FrameKind::Else, frame.block_type, resources)?;
+                    frame = self.pop_ctrl(resources)?;
                 }
-
                 for ty in results(frame.block_type, resources)? {
                     self.push_operand(ty)?;
                 }
@@ -749,7 +735,7 @@ impl OperatorValidator {
                             "type mismatch: br_table target labels have different number of types"
                         );
                     }
-                    debug_assert!(self.br_table_tmp.len() == 0);
+                    debug_assert!(self.br_table_tmp.is_empty());
                     for ty in tys.rev() {
                         let ty = self.pop_operand(Some(ty))?;
                         self.br_table_tmp.push(ty);
@@ -792,11 +778,15 @@ impl OperatorValidator {
                 let ty1 = self.pop_operand(None)?;
                 let ty2 = self.pop_operand(None)?;
                 fn is_num(ty: Option<Type>) -> bool {
-                    match ty {
-                        Some(Type::I32) | Some(Type::I64) | Some(Type::F32) | Some(Type::F64)
-                        | Some(Type::V128) | None => true,
-                        _ => false,
-                    }
+                    matches!(
+                        ty,
+                        Some(Type::I32)
+                            | Some(Type::I64)
+                            | Some(Type::F32)
+                            | Some(Type::F64)
+                            | Some(Type::V128)
+                            | None
+                    )
                 }
                 if !is_num(ty1) || !is_num(ty2) {
                     bail_op_err!("type mismatch: select only takes integral types")
@@ -1190,27 +1180,57 @@ impl OperatorValidator {
                 self.push_operand(Type::F64)?;
             }
             Operator::I32TruncSatF32S | Operator::I32TruncSatF32U => {
+                if !self.features.saturating_float_to_int {
+                    return Err(OperatorValidatorError::new(
+                        "saturating float to int conversions support is not enabled",
+                    ));
+                }
                 self.pop_operand(Some(Type::F32))?;
                 self.push_operand(Type::I32)?;
             }
             Operator::I32TruncSatF64S | Operator::I32TruncSatF64U => {
+                if !self.features.saturating_float_to_int {
+                    return Err(OperatorValidatorError::new(
+                        "saturating float to int conversions support is not enabled",
+                    ));
+                }
                 self.pop_operand(Some(Type::F64))?;
                 self.push_operand(Type::I32)?;
             }
             Operator::I64TruncSatF32S | Operator::I64TruncSatF32U => {
+                if !self.features.saturating_float_to_int {
+                    return Err(OperatorValidatorError::new(
+                        "saturating float to int conversions support is not enabled",
+                    ));
+                }
                 self.pop_operand(Some(Type::F32))?;
                 self.push_operand(Type::I64)?;
             }
             Operator::I64TruncSatF64S | Operator::I64TruncSatF64U => {
+                if !self.features.saturating_float_to_int {
+                    return Err(OperatorValidatorError::new(
+                        "saturating float to int conversions support is not enabled",
+                    ));
+                }
                 self.pop_operand(Some(Type::F64))?;
                 self.push_operand(Type::I64)?;
             }
             Operator::I32Extend16S | Operator::I32Extend8S => {
+                if !self.features.sign_extension {
+                    return Err(OperatorValidatorError::new(
+                        "sign extension operations support is not enabled",
+                    ));
+                }
                 self.pop_operand(Some(Type::I32))?;
                 self.push_operand(Type::I32)?;
             }
 
             Operator::I64Extend32S | Operator::I64Extend16S | Operator::I64Extend8S => {
+                if !self.features.sign_extension {
+                    return Err(OperatorValidatorError::new(
+                        "sign extension operations support is not enabled",
+                    ));
+                }
                 self.pop_operand(Some(Type::I64))?;
                 self.push_operand(Type::I64)?;
             }
@@ -1558,14 +1578,10 @@ impl OperatorValidator {
                 self.pop_operand(Some(Type::V128))?;
                 self.push_operand(Type::V128)?;
             }
-            Operator::F32x4FmaRelaxed
-            | Operator::F32x4FmsRelaxed
-            | Operator::F64x2FmaRelaxed
-            | Operator::F64x2FmsRelaxed
-            | Operator::F32x4MinRelaxed
-            | Operator::F32x4MaxRelaxed
-            | Operator::F64x2MinRelaxed
-            | Operator::F64x2MaxRelaxed => {
+            Operator::F32x4RelaxedMin
+            | Operator::F32x4RelaxedMax
+            | Operator::F64x2RelaxedMin
+            | Operator::F64x2RelaxedMax => {
                 self.check_relaxed_simd_enabled()?;
                 self.pop_operand(Some(Type::V128))?;
                 self.pop_operand(Some(Type::V128))?;
@@ -1726,10 +1742,10 @@ impl OperatorValidator {
                 self.pop_operand(Some(Type::V128))?;
                 self.push_operand(Type::V128)?;
             }
-            Operator::I32x4TruncSatF32x4SRelaxed
-            | Operator::I32x4TruncSatF32x4URelaxed
-            | Operator::I32x4TruncSatF64x2SZeroRelaxed
-            | Operator::I32x4TruncSatF64x2UZeroRelaxed => {
+            Operator::I32x4RelaxedTruncSatF32x4S
+            | Operator::I32x4RelaxedTruncSatF32x4U
+            | Operator::I32x4RelaxedTruncSatF64x2SZero
+            | Operator::I32x4RelaxedTruncSatF64x2UZero => {
                 self.check_relaxed_simd_enabled()?;
                 self.pop_operand(Some(Type::V128))?;
                 self.push_operand(Type::V128)?;
@@ -1741,7 +1757,11 @@ impl OperatorValidator {
                 self.pop_operand(Some(Type::V128))?;
                 self.push_operand(Type::V128)?;
             }
-            Operator::I8x16LaneSelect
+            Operator::F32x4Fma
+            | Operator::F32x4Fms
+            | Operator::F64x2Fma
+            | Operator::F64x2Fms
+            | Operator::I8x16LaneSelect
             | Operator::I16x8LaneSelect
             | Operator::I32x4LaneSelect
             | Operator::I64x2LaneSelect => {
@@ -1787,7 +1807,7 @@ impl OperatorValidator {
                 self.pop_operand(Some(Type::V128))?;
                 self.push_operand(Type::V128)?;
             }
-            Operator::I8x16SwizzleRelaxed => {
+            Operator::I8x16RelaxedSwizzle => {
                 self.check_relaxed_simd_enabled()?;
                 self.pop_operand(Some(Type::V128))?;
                 self.pop_operand(Some(Type::V128))?;
@@ -2035,7 +2055,7 @@ impl OperatorValidator {
     }
 
     pub fn finish(&mut self) -> OperatorValidatorResult<()> {
-        if self.control.len() != 0 {
+        if !self.control.is_empty() {
             bail_op_err!("control frames remain at end of function");
         }
         Ok(())
@@ -2105,20 +2125,20 @@ where
 trait PreciseIterator: ExactSizeIterator + DoubleEndedIterator {}
 impl<T: ExactSizeIterator + DoubleEndedIterator> PreciseIterator for T {}
 
-fn params<'a>(
+fn params(
     ty: TypeOrFuncType,
-    resources: &'a impl WasmModuleResources,
-) -> OperatorValidatorResult<impl PreciseIterator<Item = Type> + 'a> {
+    resources: &impl WasmModuleResources,
+) -> OperatorValidatorResult<impl PreciseIterator<Item = Type> + '_> {
     Ok(match ty {
         TypeOrFuncType::FuncType(t) => Either::A(func_type_at(resources, t)?.inputs()),
         TypeOrFuncType::Type(_) => Either::B(None.into_iter()),
     })
 }
 
-fn results<'a>(
+fn results(
     ty: TypeOrFuncType,
-    resources: &'a impl WasmModuleResources,
-) -> OperatorValidatorResult<impl PreciseIterator<Item = Type> + 'a> {
+    resources: &impl WasmModuleResources,
+) -> OperatorValidatorResult<impl PreciseIterator<Item = Type> + '_> {
     Ok(match ty {
         TypeOrFuncType::FuncType(t) => Either::A(func_type_at(resources, t)?.outputs()),
         TypeOrFuncType::Type(Type::EmptyBlockType) => Either::B(None.into_iter()),
@@ -2126,11 +2146,11 @@ fn results<'a>(
     })
 }
 
-fn label_types<'a>(
+fn label_types(
     ty: TypeOrFuncType,
-    resources: &'a impl WasmModuleResources,
+    resources: &impl WasmModuleResources,
     kind: FrameKind,
-) -> OperatorValidatorResult<impl PreciseIterator<Item = Type> + 'a> {
+) -> OperatorValidatorResult<impl PreciseIterator<Item = Type> + '_> {
     Ok(match kind {
         FrameKind::Loop => Either::A(params(ty, resources)?),
         _ => Either::B(results(ty, resources)?),
